<?php

namespace Drupal\remove_broken_missing_blocks\Drush\Commands;

use Consolidation\OutputFormatters\Options\FormatterOptions;
use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Consolidation\OutputFormatters\Transformations\TableTransformation;
use Drupal\block\Entity\Block;
use Drupal\Core\Block\Plugin\Block\Broken;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Token;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush commandfile.
 */
final class BlocksCommands extends DrushCommands {
  use StringTranslationTrait;
  /**
   * Constructs a BlocksCommands object.
   */
  public function __construct(
    private readonly Token $token,
    private readonly Connection $connection,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('token'),
      $container->get('database'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Command description here.
   */
  #[CLI\Command(name: 'rbmb:delete-broken-missing-blocks', aliases: ['dbmb'])]
  #[CLI\Option(name: 'deleteNoStorage', description: 'Deletes config when no storage found (default 0).', suggestedValues: [0, 1])]
  #[CLI\Option(name: 'deleteBroken', description: 'Deletes config when broken/missing block found (default 0).', suggestedValues: [0, 1])]
  #[CLI\Option(name: 'onlyWithTroubles', description: 'Only shows blocks with troubles (default 1).', suggestedValues: [0, 1])]
  #[CLI\Usage(name: 'rbmb:delete-broken-blocks --deleteNoStorage 0  --deleteBroken 0', description: 'removes all broken/missing blocks in a website instance (0 means only shows results)')]
  #[CLI\FieldLabels(labels: [
    'config_name' => 'Config name',
    'status' => 'Status',
    'action' => 'Action'
  ])]
  #[CLI\DefaultTableFields(fields: ['config_name', 'status', 'action'])]
  #[CLI\FilterDefaultField(field: 'config_name')]
  public function deleteBrokenMissingBlocks($options = ['deleteNoStorage' => 0, 'deleteBroken' => 0, 'onlyWithTroubles' => 1]) {
    $rows = [];
    $select = $this->connection->select('config', 'c');
    $select->fields('c', ['name']);
    $select->condition('c.name', 'block.block.%', 'LIKE');
    $result = $select->execute()->fetchAll();
    foreach($result as $configName) {
      $block = $this->configFactory->getEditable($configName->name);
      $row = ['config_name' => $configName->name, 'status' => FALSE, 'action' => FALSE];
      if ($block) {
        $blockConfigStorage = $this->entityTypeManager->getStorage('block')
          ->loadByProperties(['uuid' => $block->get('uuid')]);
        if (!$blockConfigStorage) {
          if ($options['deleteNoStorage']) {
            $row['action'] = $this->t('Deleted');
            $block->delete();
            $this->connection->delete('config')
              ->condition('name', $configName->name)
              ->execute();
          }
          $row['status'] = $this->t('No storage found.');
          if ($options['verbose']) {
            $this->logger()
              ->success("No config found : '@config' has been deleted.", [
                '@config' => $configName->name
              ]);
          }
        } else if (isset($blockConfigStorage[$block->get('id')]) && $blockConfigStorage[$block->get('id')] instanceof Block) {
          $definitions = $blockConfigStorage[$block->get('id')]->getPlugin()
            ->getPluginDefinition();
          if (isset($definitions['class']) && $definitions['class'] === Broken::class) {
            if ($options['deleteBroken']) {
              $row['action'] = $this->t('Deleted');
              $block->delete();
              $this->connection->delete('config')->condition('name', $configName->name)->execute();
            }
            $row['status'] = $this->t('Broken/Missing');
            if ($options['verbose']) {
              $this->logger()
                ->success("Broken config found : '@config' should be deleted", [
                  '@config' => $configName->name
                ]);
            }
          }
        }
        $row['action'] = $row['action'] ?: $this->t('Nothing done');

        $hasTrouble = (bool) $row['status'];
        $row['status'] = $row['status'] ?: $this->t('OK');
        if ($options['verbose']) {
          $this->logger->notice("Config still in use : '@config' did not move.");
        }
        if ($hasTrouble ||  !$options['onlyWithTroubles']) {
          $rows[] = $row;
        }
      }
    }
    return new RowsOfFields($rows, [FormatterOptions::TERMINAL_WIDTH => '1000', FormatterOptions::LIST_ORIENTATION => TRUE]);;
  }
}
